from django.conf import settings
from django.db.models import BooleanField, Count, ExpressionWrapper, Q
from django.http.response import HttpResponseForbidden
from django.views.generic.base import TemplateView

from core.choices import UserBadgeVisibility
from core.models import Badge, Conference, ConferenceMember, PlatformUser, Project, Room, UserBadge
from core.models.assemblies import Assembly
from core.models.events import Event
from core.models.ticket import ConferenceMemberTicket


class MetricsView(TemplateView):
    template_name = 'metrics/metrics.txt'
    content_type = 'text/plain; charset=utf-8'

    def dispatch(self, request, *args, **kwargs):
        """
        Only allow IP addresses that are listed in settings.METRICS_SERVER_IPS.
        """
        remote_addr = request.META.get('REMOTE_ADDR')
        if remote_addr in settings.METRICS_SERVER_IPS or '*' in settings.METRICS_SERVER_IPS:
            return super().dispatch(request, *args, **kwargs)
        else:
            return HttpResponseForbidden('This IP address is not allowed to access metrics (see setting METRICS_SERVER_IPS).')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        metrics = {
            'hub_conferences_total': {
                'help': 'total number of conferences',
                'type': 'counter',
                'value': Conference.objects.count(),
            },
            'hub_users_total': {
                'help': 'total number of users',
                'type': 'counter',
                'values': {
                    '{user_type="human"}': PlatformUser.objects.filter(user_type=PlatformUser.Type.HUMAN).count(),
                    '{user_type="service"}': PlatformUser.objects.filter(user_type=PlatformUser.Type.SERVICE).count(),
                    '{user_type="bot"}': PlatformUser.objects.filter(user_type=PlatformUser.Type.BOT).count(),
                    '{user_type="assembly"}': PlatformUser.objects.filter(user_type=PlatformUser.Type.ASSEMBLY).count(),
                },
            },
            'hub_conference_members': {'help': 'members in the conference', 'type': 'counter', 'values': {}},
            'hub_conference_members_staff': {'help': 'staff count', 'type': 'counter', 'values': {}},
            'hub_conference_members_themes': {'help': 'used themes by members in the conference', 'type': 'gauge', 'values': {}},
            'hub_conference_tickets': {'help': 'registered tickets', 'type': 'counter', 'values': {}},
            'hub_conference_assemblies': {'help': "conference's assemblies", 'type': 'gauge', 'values': {}},
            'hub_conference_badges_public': {'help': "conference's badges (public)", 'type': 'gauge', 'values': {}},
            'hub_conference_badges_hidden': {'help': "conference's badges (non-public)", 'type': 'gauge', 'values': {}},
            'hub_conference_badges_accepted': {'help': "conference's badges (accepted/assigned)", 'type': 'gauge', 'values': {}},
            'hub_conference_badges_redeemed': {'help': "conference's badges (redeemed, but not accepted yet)", 'type': 'gauge', 'values': {}},
            'hub_conference_events': {'help': "conference's events", 'type': 'gauge', 'values': {}},
            'hub_conference_rooms': {'help': "conference's rooms", 'type': 'gauge', 'values': {}},
            'hub_conference_projects': {'help': "conference's projects", 'type': 'gauge', 'values': {}},
        }

        for conference in Conference.objects.filter(is_public=True):
            slug = conference.slug

            # hub_conference_assemblies
            asm_count = Assembly.objects.filter(conference=conference).values('state').annotate(count=Count('*'))
            expected_states = list(Assembly.State.values)
            for line in asm_count:
                metrics[f'hub_conference_{'assemblies'}']['values'][f'{{conference="{conference.slug}",state="{line["state"]}"}}'] = line['count']
                expected_states.remove(line['state'])
            for missing in expected_states:
                metrics[f'hub_conference_{'assemblies'}']['values'][f'{{conference="{conference.slug}",state="{missing}"}}'] = 0

            # hub_conference_rooms
            room_counts = {room_type: 0 for room_type in Room.RoomType.values}
            for room_type, cnt in (
                Room.objects.filter(conference=conference).values('room_type').annotate(cnt=Count('room_type')).values_list('room_type', 'cnt')
            ):
                room_counts[room_type] = cnt

            for room_type, room_count in room_counts.items():
                metrics['hub_conference_rooms']['values'][f'{{conference="{slug}",room_type="{room_type}"}}'] = room_count

            # hub_conference_projects
            is_sos_query = ExpressionWrapper(Q(assembly=None), output_field=BooleanField())
            project_counts = (
                Project.objects.filter(conference=conference).annotate(is_sos=is_sos_query).values('is_sos', 'is_public').annotate(count=Count('*'))
            )
            for line in project_counts:
                is_sos = line['is_sos']
                is_public = line['is_public']
                metrics['hub_conference_projects']['values'][f'{{conference="{slug}",self_organized="{is_sos}",public="{is_public}"}}'] = line['count']

            # hub_conference_events
            for kind in Event.Kind.values:
                event_count = Event.objects.filter(conference=conference, kind=Event.Kind(kind)).count()
                metrics['hub_conference_events']['values'][f'{{conference="{slug}",kind="{kind}"}}'] = event_count

            # hub_conference_badges_{public,private}
            for visibility in (Badge.State.HIDDEN, Badge.State.PUBLIC):
                metrics['hub_conference_badges_' + visibility]['values'][f'{{conference="{slug}",badge_type="achievement"}}'] = Badge.objects.filter(
                    conference=conference, state=visibility
                ).count()
                metrics['hub_conference_badges_' + visibility]['values'][f'{{conference="{slug}",badge_type="sticker"}}'] = Badge.objects.filter(
                    conference=conference, state=visibility
                ).count()

            # hub_conference_badges_{accepted,redeemed}
            for userbadge_visibility in UserBadgeVisibility.values:
                metrics['hub_conference_badges_accepted']['values'][f'{{conference="{slug}",visibility="{userbadge_visibility}"}}'] = UserBadge.objects.filter(
                    badge__conference=conference, visibility=userbadge_visibility, accepted_by_user=True
                ).count()
                metrics['hub_conference_badges_redeemed']['values'][f'{{conference="{slug}",visibility="{userbadge_visibility}"}}'] = UserBadge.objects.filter(
                    badge__conference=conference, visibility=userbadge_visibility, accepted_by_user=False
                ).count()

            # hub_conference_members{,_staff}
            metrics['hub_conference_members']['values'][f'{{conference="{slug}",active_angel="true"}}'] = ConferenceMember.objects.filter(
                conference=conference, active_angel=True
            ).count()
            metrics['hub_conference_members']['values'][f'{{conference="{slug}",active_angel="false"}}'] = ConferenceMember.objects.filter(
                conference=conference, active_angel=False
            ).count()
            metrics['hub_conference_members_staff']['values'][f'{{conference="{slug}"}}'] = ConferenceMember.objects.filter(
                conference=conference, is_staff=True
            ).count()

            # hub_conference_members_themes
            for theme in PlatformUser.Theme.values:
                metrics['hub_conference_members_themes']['values'][f'{{conference="{slug}",theme="{theme}"}}'] = ConferenceMember.objects.filter(
                    conference=conference, user__theme=theme
                ).count()

            # hub_conference_tickets
            metrics['hub_conference_tickets']['values'][f'{{conference="{slug}"}}'] = ConferenceMemberTicket.objects.filter(conference=conference).count()

        context['metrics'] = metrics

        return context
