import logging

from django import forms
from django.contrib import messages
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from core.base_forms import TranslatedFieldsForm
from core.integrations import Hangar, IntegrationError
from core.models import (
    Room,
    RoomLink,
)

logger = logging.getLogger(__name__)


class AssemblyRoomCreateForm(forms.Form):
    name = forms.CharField(max_length=200)

    def __init__(self, *args, assembly, **kwargs):
        self.assembly = assembly
        super().__init__(*args, **kwargs)

    def clean(self, *args, **kwargs):
        cleaned_data = super().clean(*args, **kwargs)
        if 'name' not in cleaned_data:
            return cleaned_data

        if self.assembly.rooms.filter(name=cleaned_data['name']).exists():
            raise ValidationError({'name': _('Room__duplicate_name')})

        return cleaned_data

    def create_room(self, request):
        raise NotImplementedError('AssemblyCreateRoomForm.create_room() must be overridden')

    def _create_room(self, room_type, backend_status=Room.BackendStatus.NONE, **kwargs):
        room = self.assembly.rooms.create(
            conference=self.assembly.conference,
            room_type=room_type,
            backend_status=backend_status,
            **kwargs,
        )
        self.room_id = room.id

        return room

    def create_room_on_backend(self, request, integration, room):
        if integration.is_available():
            try:
                integration.create_room(room)

            except IntegrationError as err:
                logger.error('Got integration error for room %s: %s', room, err)
                messages.warning(request, 'The backend returned an error, your room was NOT created:' + str(err))
                return False

        else:
            messages.warning(request, 'BBB is currently not available, your room will be created later.')

        return True


class AssemblyRoomCreateGenericForm(AssemblyRoomCreateForm):
    room_type = forms.ChoiceField(
        choices=[c for c in Room.RoomType.choices if c[0] not in [*Room.BACKEND_ROOMTYPES, Room.RoomType.PROJECT]],
        disabled=True,
    )
    capacity = forms.IntegerField(min_value=0)

    def create_room(self, request):
        return self._create_room(**self.cleaned_data)


class AssemblyRoomCreateBigBlueButtonForm(AssemblyRoomCreateForm):
    capacity = forms.IntegerField(min_value=0, initial=0, label=_('Room__bigbluebutton_capacity'), help_text=_('Room__bigbluebutton_capacity__help'))

    def create_room(self, request):
        room = self._create_room(
            room_type=Room.RoomType.BIGBLUEBUTTON,
            backend_status=Room.BackendStatus.NEW,
            **self.cleaned_data,
        )

        # not creating a room on the backend as rooms will be created on first join.
        # If empty, they time out after some minutes anyways and therefore get recreated regularly.

        return room


class AssemblyRoomCreateHangarForm(AssemblyRoomCreateForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['name'].initial = self.assembly.slug
        self.fields['name'].disabled = True

    def clean(self, *args, **kwargs):
        cleaned_data = super().clean(*args, **kwargs)

        if not Hangar.can_create_for_assembly(self.assembly):
            raise ValidationError(_('Room__new-hangar__unavailable'))

        return cleaned_data

    def create_room(self, request):
        room = self.assembly.rooms.create(
            conference=self.assembly.conference,
            room_type=Room.RoomType.HANGAR,
            backend_status=Room.BackendStatus.NEW,
            **self.cleaned_data,
        )
        room.save()
        self.room_id = room.id

        if not self.create_room_on_backend(request, Hangar, room):
            room.delete()
            room = None
            self.room_id = None

        return room


class AssemblyRoomEditForm(TranslatedFieldsForm):
    class Meta:
        model = Room
        fields = ['name', 'slug', 'description', 'is_public_fahrplan', 'is_official', 'official_room_order', 'capacity']

    def __init__(self, *args, with_capacity: bool = False, schedule_admin: bool = False, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['slug'].disabled = True
        if self.instance.room_type in Room.BACKEND_ROOMTYPES:
            self.fields['backend_status'] = forms.CharField(initial=self.instance.get_backend_status_display(), disabled=True)
        if self.instance.room_type not in Room.TYPES_WITH_CAPACITY:
            del self.fields['capacity']
        elif self.instance.room_type == Room.RoomType.BIGBLUEBUTTON:
            self.fields['capacity'].label = _('Room__bigbluebutton_capacity')
            self.fields['capacity'].help_text = _('Room__bigbluebutton_capacity__help')

            if self.instance.backend_status == Room.BackendStatus.ACTIVE:
                self.fields['capacity'].help_text += ' ' + _('Room__bigbluebutton__active_room')
                self.fields['capacity'].disabled = True

        if self.instance.room_type in [Room.RoomType.HANGAR]:
            self.fields['name'].disabled = True

        if not schedule_admin:
            del self.fields['is_public_fahrplan']
            del self.fields['is_official']
            del self.fields['official_room_order']

    def clean_name(self):
        if Room.objects.filter(assembly=self.instance.assembly, name=self.cleaned_data['name']).exclude(pk=self.instance.pk).exists():
            raise ValidationError(_('Room__name-assembly-unique'))

        # update slug to be based on the new name iff the name changed.
        if self.instance.name != self.cleaned_data['name']:
            self.instance.name = self.cleaned_data['name']
            self.instance.generate_slug()
            self.cleaned_data['slug'] = self.instance.slug
        return self.cleaned_data['name']

    def clean_slug(self):
        # self.instance.slug contains the new slug that was computed by `clean_name`.
        # We return this here so that it doesn't get overwritten again by the initial slug value because of `self.fields['slug'].disabled = True`
        return self.instance.slug

    def clean_capacity(self):
        capacity = self.cleaned_data.get('capacity')
        if capacity is None:
            return None

        try:
            capacity = int(capacity)
        except ValueError:
            raise ValidationError(_('Room__capacity-invalid'))

        if capacity < 0:
            raise ValidationError(_('Room__capacity-negative'))
        return capacity

    def save(self, commit=False):
        obj = super().save(commit)

        if obj.room_type in Room.TYPES_WITH_CAPACITY:
            if (capacity := self.cleaned_data.get('capacity')) is not None:
                obj.capacity = int(capacity)

        if commit:
            obj.save()

        return obj


class AssemblyRoomEditHangarForm(TranslatedFieldsForm):
    class Meta:
        model = Room
        fields = ['backend_status', 'backend_link']

    def __init__(self, *args, with_capacity: bool = False, schedule_admin: bool = False, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['backend_status'].disabled = True
        self.fields['backend_link'].label = _('Room__hangar_backend_link')
        self.fields['backend_link'].help_text = _('Room__hangar_backend_link__help')
        self.fields['backend_link'].disabled = True


class AssemblyRoomLinkCreateForm(forms.ModelForm):
    class Meta:
        model = RoomLink
        fields = ['name', 'link_type', 'link']
