from django.urls import reverse
from django.utils.translation import activate
from django.utils.translation import gettext as _

from core.forms.teams import TeamForm
from core.models import (
    PlatformUser,
    Team,
    TeamMember,
)

from backoffice.tests.base import BackOfficeTestCase


class TeamListViewTestCase(BackOfficeTestCase):
    users = ['admin', 'staff', 'user', 'team_manager', 'team_user']
    detail_matrix = {
        Team.TeamVisibility.PUBLIC: ['admin', 'staff', 'user', 'team_manager', 'team_user'],
        Team.TeamVisibility.STAFF: ['admin', 'staff', 'team_manager', 'team_user'],
        Team.TeamVisibility.HIDDEN: ['admin', 'team_manager', 'team_user'],
    }
    create_button = ['admin']

    def setUp(self):
        super().setUp()
        self.team_user = PlatformUser.objects.create_user(username='team_user')
        self.team_manager = PlatformUser.objects.create_user(username='team_manager')
        self.teams = {}
        self.team = Team.objects.create(
            name='team',
            conference=self.conf,
        )
        TeamMember.objects.create(team=self.team, user=self.team_manager, can_manage=True)
        TeamMember.objects.create(team=self.team, user=self.team_user)

    def test_team_list_unauthenicated(self):
        activate('en')
        for visibility in Team.TeamVisibility:
            with self.subTest(visibility=visibility):
                self.team.visibility = visibility
                self.team.save()
                response = self.client.get(reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
                self.assertRedirects(response, reverse('backoffice:login') + '?next=' + reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))

    def test_team_list_admin(self):
        activate('en')
        for user in self.users:
            for visibility, users in self.detail_matrix.items():
                with self.subTest(user=user, visibility=visibility):
                    self.team.visibility = visibility
                    self.team.save()
                    self.client.force_login(getattr(self, user))
                    response = self.client.get(reverse('backoffice:teams'))
                    self.assertEqual(response.status_code, 200)
                    self.assertTemplateUsed(response, 'backoffice/teams/list.html')
                    if user in self.create_button:
                        self.assertContains(response, _('Team__create__button'))
                    else:
                        self.assertNotContains(response, _('Team__create__button'))
                    if user in users:
                        self.assertIn(self.team, response.context['teams'])
                    else:
                        self.assertNotIn(self.team, response.context['teams'])


class TeamDetailViewTestCase(BackOfficeTestCase):
    detail_matrix = {
        Team.TeamVisibility.PUBLIC: {
            'admin': ['edit', 'delete', 'join'],
            'staff': ['join'],
            'team_manager': ['edit', 'leave'],
            'user': ['leave'],
            'user_2': ['join'],
        },
        Team.TeamVisibility.STAFF: {
            'admin': ['edit', 'delete', 'join'],
            'staff': ['join'],
            'team_manager': ['edit', 'leave'],
            'user': ['leave'],
            'user_2': False,
        },
        Team.TeamVisibility.HIDDEN: {
            'admin': ['edit', 'delete', 'join'],
            'staff': False,
            'team_manager': ['edit', 'leave'],
            'user': ['leave'],
            'user_2': False,
        },
    }
    button_texts = {
        'edit': _('Team__edit__button'),
        'delete': _('Team__delete__button'),
        'join': _('TeamMember__join'),
        'leave': _('TeamMember__leave'),
    }

    def setUp(self):
        super().setUp()
        self.team_manager = PlatformUser.objects.create_user(
            username='team_mamanger',
        )
        self.user_2 = PlatformUser.objects.create_user(
            username='user2',
        )
        self.team = Team.objects.create(
            name='team',
            conference=self.conf,
        )
        self.team_member_manager = TeamMember.objects.create(team=self.team, user=self.team_manager, can_manage=True)
        self.team_member_user = TeamMember.objects.create(team=self.team, user=self.user)

    def test_team_detail_unauthenicated(self):
        activate('en')
        for visibility in Team.TeamVisibility:
            with self.subTest(visibility=visibility):
                self.team.visibility = visibility
                self.team.save()
                response = self.client.get(reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
                self.assertRedirects(response, reverse('backoffice:login') + '?next=' + reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))

    def test_team_detail(self):
        activate('en')
        for visibility, matrix in self.detail_matrix.items():
            for user, buttons in matrix.items():
                with self.subTest(visibility=visibility, user=user):
                    self.team.visibility = visibility
                    self.team.save()
                    self.client.force_login(getattr(self, user))
                    response = self.client.get(reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
                    if buttons is False:
                        self.assertEqual(response.status_code, 403)
                    else:
                        self.assertEqual(response.status_code, 200)
                        self.assertTemplateUsed(response, 'backoffice/teams/detail.html')
                        self.assertEqual(self.team, response.context['team'])
                        for button, text in self.button_texts.items():
                            if button not in buttons:
                                self.assertNotContains(response, text)
                            else:
                                self.assertContains(response, text)


class TeamCreateViewTestCase(BackOfficeTestCase):
    expected_results = {
        'admin': True,
        'staff': False,
        'user': False,
    }

    def test_team_create_unauthenicated(self):
        activate('en')
        response = self.client.get(reverse('backoffice:team-create'))
        self.assertRedirects(response, reverse('backoffice:login') + '?next=' + reverse('backoffice:team-create'))

    def test_team_create(self):
        for user, result in self.expected_results.items():
            with self.subTest(user=user):
                self.client.force_login(getattr(self, user))
                response = self.client.get(reverse('backoffice:team-create'))
                if result is True:
                    self.assertEqual(response.status_code, 200)
                    self.assertTemplateUsed(response, 'backoffice/teams/create_edit.html')
                    self.assertContains(response, _('create'))
                    self.assertContains(response, _('Team__create__title'))

                    response = self.client.post(reverse('backoffice:team-create'), data={'name': 'team', 'visibility': Team.TeamVisibility.PUBLIC})
                    team = Team.objects.filter(name='team', visibility=Team.TeamVisibility.PUBLIC).first()
                    self.assertIsNotNone(team, 'Team not created')
                    self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': team.uuid}))
                else:
                    self.assertEqual(response.status_code, 403)


class TeamUpdateViewTestCase(BackOfficeTestCase):
    fields = {
        'name': 'test-name',
        'description_en': 'test-en',
        'description_de': 'test-de',
        'visibility': Team.TeamVisibility.PUBLIC.value,
    }
    expected_results = {
        'admin': ['name', 'description_en', 'description_de', 'visibility'],
        'staff': False,
        'team_manager': ['name', 'description_en', 'description_de'],
        'user': False,
    }

    def setUp(self):
        super().setUp()
        self.team_manager = PlatformUser.objects.create_user(username='team_mamanger')
        self.user_2 = PlatformUser.objects.create_user(username='user2')

    def repeatable_setUp(self):
        for team in Team.objects.all():
            team.delete()
        self.team = Team.objects.create(
            name='team',
            conference=self.conf,
        )
        self.team_member_manager = TeamMember.objects.create(team=self.team, user=self.team_manager, can_manage=True)
        self.team_member_user = TeamMember.objects.create(team=self.team, user=self.user)

    def test_all_fields(self):
        self.assertListEqual(list(self.fields.keys()), TeamForm.Meta.fields)

    def test_team_edit(self):
        activate('en')
        for visibility in Team.TeamVisibility:
            for user, user_fields in self.expected_results.items():
                with self.subTest(visibility=visibility, user=user):
                    self.repeatable_setUp()
                    self.team.visibility = visibility
                    self.team.save()
                    self.client.force_login(getattr(self, user))
                    response = self.client.get(reverse('backoffice:team-edit', kwargs={'uuid': self.team.uuid}))
                    if user_fields:
                        self.assertEqual(response.status_code, 200)
                        self.assertTemplateUsed(response, 'backoffice/teams/create_edit.html')
                        self.assertNotContains(response, _('create'))
                        self.assertContains(response, _('Team__edit__submit'))
                        for field in self.fields:
                            if field in user_fields:
                                self.assertIn(field, response.context['form'].fields)
                            else:
                                self.assertNotIn(field, response.context['form'].fields)
                    else:
                        self.assertEqual(response.status_code, 403)
                    response = self.client.post(
                        reverse('backoffice:team-edit', kwargs={'uuid': self.team.uuid}),
                        data={field: self.fields[field] for field in user_fields} if user_fields else {'name': self.fields['name']},
                    )
                    if user_fields:
                        self.team.refresh_from_db()
                        self.assertRedirects(response, reverse('backoffice:team', kwargs={'uuid': self.team.uuid}))
                        for field in [x for x in self.fields if x in user_fields]:
                            self.assertEqual(getattr(self.team, field), self.fields[field])
                    else:
                        self.assertEqual(response.status_code, 403)


class TeamDeleteViewTestCase(BackOfficeTestCase):
    expected_results = {
        'admin': True,
        'staff': False,
        'team_manager': False,
        'user': False,
    }

    def setUp(self):
        super().setUp()
        self.team_manager = PlatformUser.objects.create_user(username='team_mamanger')

    def repeatable_setUp(self, visibility=Team.TeamVisibility.STAFF):
        if team := Team.objects.filter(name='team').first():
            team.delete()
        self.team = Team.objects.create(
            name='team',
            conference=self.conf,
            visibility=visibility,
        )
        self.team_member_manager = TeamMember.objects.create(team=self.team, user=self.staff, can_manage=True)
        self.team_member_user = TeamMember.objects.create(team=self.team, user=self.user)

    def test_team_delete_unauthenicated(self):
        self.repeatable_setUp()
        for visibility in Team.TeamVisibility:
            with self.subTest(visibility=visibility):
                self.team.visibility = visibility
                self.team.save()
                response = self.client.get(reverse('backoffice:team-delete', kwargs={'uuid': self.team.uuid}))
                self.assertRedirects(response, reverse('backoffice:login') + '?next=' + reverse('backoffice:team-delete', kwargs={'uuid': self.team.uuid}))

    def test_team_delete_admin(self):
        activate('en')
        for user, result in self.expected_results.items():
            for visibility in Team.TeamVisibility:
                with self.subTest(user=user, visibility=visibility):
                    self.repeatable_setUp(visibility)
                    self.client.force_login(getattr(self, user))
                    response = self.client.get(reverse('backoffice:team-delete', kwargs={'uuid': self.team.uuid}))
                    if result is True:
                        self.assertEqual(response.status_code, 200)
                        self.assertTemplateUsed(response, 'backoffice/components/confirmation_modal.html')
                        self.assertContains(response, _('Team__delete__warning__header'))
                        self.assertContains(response, _('Team__delete__submit'))
                    else:
                        self.assertEqual(response.status_code, 403)

                    response = self.client.post(reverse('backoffice:team-delete', kwargs={'uuid': self.team.uuid}))
                    if result is True:
                        # No confirmation, should not delete
                        self.assertTrue(Team.objects.filter(name='team').exists())
                        self.assertEqual(response.status_code, 200)
                        self.assertTemplateUsed(response, 'backoffice/components/confirmation_modal.html')
                        self.assertContains(response, _('Team__delete__warning__header'))
                        self.assertContains(response, _('Team__delete__submit'))
                    else:
                        self.assertEqual(response.status_code, 403)

                    response = self.client.post(reverse('backoffice:team-delete', kwargs={'uuid': self.team.uuid}), data={'confirmation': 'true'})
                    if result is True:
                        self.assertFalse(Team.objects.filter(name='team').exists())
                        self.assertRedirects(response, reverse('backoffice:teams'))
                    else:
                        self.assertEqual(response.status_code, 403)
