from typing import Any

from django.contrib.auth.views import RedirectURLMixin
from django.contrib.messages import SUCCESS, add_message
from django.core.exceptions import BadRequest
from django.forms.models import BaseModelForm
from django.shortcuts import get_object_or_404
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.generic import CreateView

from core.forms import InvitationHabitatForm, InvitationTeamForm
from core.models import Assembly, Invitation, PlatformUser, Team

from backoffice.views.mixins import ConferenceLoginRequiredMixin


class InvitationCreateView(ConferenceLoginRequiredMixin, RedirectURLMixin, CreateView):
    template_name = 'backoffice/invitations/create_edit.html'
    model = Invitation

    @property
    def invitation_type(self):
        if (invitation_type := str(self.kwargs.get('type', '')).upper()) not in Invitation.InvitationType.values:
            raise BadRequest('Invalid invitation type')
        return invitation_type

    def has_permission(self):
        match self.invitation_type:
            case Invitation.InvitationType.HABITAT:
                assembly = get_object_or_404(Assembly, pk=self.kwargs.get('requester_id'))
                return self.request.user.has_perm('core.change_assembly', assembly)
            case Invitation.InvitationType.TEAM_TO_MEMBER:
                team = get_object_or_404(Team, uuid=self.kwargs.get('requester_id'))
                return self.request.user.has_perm('core.change_team', team)
            case Invitation.InvitationType.MEMBER_TO_TEAM:
                user = get_object_or_404(PlatformUser, uuid=self.kwargs.get('requester_id'))
                return user == self.request.user
            case _:  # pragma: no cover
                raise NotImplementedError('Invitation type not implemented')

    def get_form_class(self) -> type[BaseModelForm]:
        match self.invitation_type:
            case Invitation.InvitationType.HABITAT:
                return InvitationHabitatForm
            case Invitation.InvitationType.TEAM_TO_MEMBER | Invitation.InvitationType.MEMBER_TO_TEAM:
                return InvitationTeamForm
            case '_':  # pragma: no cover
                raise NotImplementedError('Invalid invitation type')

    def get_form_kwargs(self) -> dict[str, Any]:
        kwargs = {
            **super().get_form_kwargs(),
            'conference': self.conference,
            'user': self.request.user,
            'requester_id': self.kwargs.get('requester_id'),
            'requested_id': self.kwargs.get('requested_id'),
        }
        if self.invitation_type in [Invitation.InvitationType.TEAM_TO_MEMBER, Invitation.InvitationType.MEMBER_TO_TEAM]:
            kwargs['from_team'] = self.invitation_type == Invitation.InvitationType.TEAM_TO_MEMBER
        return kwargs

    def form_valid(self, form):
        response = super().form_valid(form)
        add_message(self.request, SUCCESS, f'Invitation to {self.object.requested.name} sent')
        return response

    def get_default_redirect_url(self) -> str:
        return reverse('backoffice:invitation-detail', kwargs={'pk': self.object.pk})

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        match self.invitation_type:
            case Invitation.InvitationType.HABITAT:
                context.update(
                    {
                        'introduction_text': _('Invitation__introduction__habitat'),
                        'invitation__type': _('Invitation__type__habitat'),
                    }
                )

            case Invitation.InvitationType.TEAM_TO_MEMBER:
                context.update(
                    {
                        'introduction_text': _('Invitation__introduction__team'),
                        'invitation__type': _('Invitation__type__team_to_member'),
                    }
                )
            case Invitation.InvitationType.MEMBER_TO_TEAM:
                context.update(
                    {
                        'introduction_text': _('Invitation__introduction__team'),
                        'invitation__type': _('Invitation__type__member_to_team'),
                    }
                )
            case _:  # pragma: no cover
                raise NotImplementedError('Invalid invitation type')
        return context
