from typing import Any

from django.contrib.auth.views import RedirectURLMixin
from django.contrib.messages import ERROR, SUCCESS, WARNING, add_message
from django.core.exceptions import BadRequest
from django.forms.models import BaseModelForm
from django.http import HttpRequest, HttpResponseRedirect
from django.http.response import HttpResponse
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.generic import UpdateView, View
from django.views.generic.detail import SingleObjectMixin

from core.forms import InvitationHabitatForm, InvitationTeamForm
from core.models import Invitation

from backoffice.views.mixins import ConferenceLoginRequiredMixin, ConferenceRuleLoginRequiredMixin


class InvitationUpdateView(ConferenceRuleLoginRequiredMixin, UpdateView):
    model = Invitation
    template_name = 'backoffice/invitations/create_edit.html'
    permission_required = 'core.change_invitation'

    def get_invitation(self) -> Invitation:
        invitation = self.get_object()
        if not Invitation.type_is(invitation):  # pragma: no cover
            raise ValueError('Invalid object type')
        return invitation

    def get_form_class(self) -> type[BaseModelForm]:
        invitation = self.get_invitation()
        match invitation.type:
            case Invitation.InvitationType.HABITAT:
                return InvitationHabitatForm
            case Invitation.InvitationType.TEAM_TO_MEMBER | Invitation.InvitationType.MEMBER_TO_TEAM:
                return InvitationTeamForm
            case _:  # pragma: no cover
                raise NotImplementedError('Invalid invitation type')

    def get_context_data(self, **kwargs):
        invitation = self.get_invitation()
        context = super().get_context_data(**kwargs)
        context['active_page'] = 'invitations'
        match invitation.type:
            case Invitation.InvitationType.HABITAT:
                context.update(
                    {
                        'introduction_text': _('Invitation__introduction__habitat'),
                        'invitation__type': _('Invitation__type__habitat'),
                    }
                )
            case Invitation.InvitationType.TEAM_TO_MEMBER:
                context.update(
                    {
                        'invitation__type': _('Invitation__type__team_to_member'),
                        'introduction_text': _('Invitation__introduction__team'),
                    }
                )
            case Invitation.InvitationType.MEMBER_TO_TEAM:
                context.update(
                    {
                        'invitation__type': _('Invitation__type__member_to_team'),
                        'introduction_text': _('Invitation__introduction__team'),
                    }
                )
            case _:  # pragma: no cover
                raise NotImplementedError('Invalid invitation type')
        return context

    def get_form_kwargs(self) -> dict[str, Any]:
        return {
            **super().get_form_kwargs(),
            'user': self.request.user,
            'conference': self.conference,
        }

    def post(self, request: HttpRequest, *args: str, **kwargs: Any) -> HttpResponse:
        invitation = self.get_invitation()
        response = super().post(request, *args, **kwargs)
        add_message(
            request,
            SUCCESS,
            _('Invitation__updated__message %(requester)s %(requested)s')
            % {
                'requester': invitation.requester.name,
                'requested': invitation.requested.name,
            },
        )
        return response

    def get_success_url(self) -> str:
        return reverse('backoffice:invitation-detail', kwargs={'pk': self.object.pk})


class InvitationDecisionView(ConferenceLoginRequiredMixin, RedirectURLMixin, SingleObjectMixin, View):
    http_method_names = ['post']
    model = Invitation

    def post(self, request: HttpRequest, *args: str, **kwargs: Any) -> HttpResponse:
        self.object = self.get_object()  # pylint: disable=attribute-defined-outside-init
        if not Invitation.type_is(invitation := self.object):  # pragma: no cover
            raise ValueError('Invalid object type')
        action = request.POST.get('action', '')
        # Check permissions
        match action:
            case 'withdraw':
                if not request.user.has_perm('core.change_invitation', invitation):
                    return self.handle_no_permission()
            case 'accept' | 'reject':
                if not request.user.has_perm('core.accept_invitation', invitation):
                    return self.handle_no_permission()
            case _:
                raise BadRequest('Invalid action')
        match action:
            case 'withdraw':
                invitation.withdraw(request.user)
                add_message(
                    request,
                    WARNING,
                    _('Invitation__withdraw__message %(requester)s %(requested)s')
                    % {
                        'requester': invitation.requester.name,
                        'requested': invitation.requested.name,
                    },
                )
                return HttpResponseRedirect(self.get_success_url())
            case 'accept':
                invitation.accept(request.user)
                add_message(
                    request,
                    SUCCESS,
                    _('Invitation__accept__message %(requester)s %(requested)s')
                    % {
                        'requester': invitation.requester.name,
                        'requested': invitation.requested.name,
                    },
                )
                return HttpResponseRedirect(self.get_success_url())
            case 'reject':
                invitation.reject(request.user)
                add_message(
                    request,
                    ERROR,
                    _('Invitation__rejected__message %(requester)s %(requested)s')
                    % {
                        'requester': invitation.requester.name,
                        'requested': invitation.requested.name,
                    },
                )
                return HttpResponseRedirect(self.get_success_url())
            case _:  # pragma: no cover
                raise BadRequest('Invalid action')

    def get_default_redirect_url(self) -> str:
        return reverse('backoffice:invitation-detail', kwargs={'pk': self.object.pk})
