import logging

from django.contrib import messages
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _
from django.views.generic import ListView
from django.views.generic.detail import SingleObjectTemplateResponseMixin
from django.views.generic.edit import CreateView, ModelFormMixin, UpdateView

from core.models.map import MapFloor, MapPOI

from backoffice.forms import POIForm
from backoffice.views.mixins import ConferenceLoginRequiredMixin, guess_active_sidebar_item

logger = logging.getLogger(__name__)


class MapAdminMixin(ConferenceLoginRequiredMixin):
    require_conference = True
    permission_required = ['core.map_edit']

    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        ctx['active_page'] = 'map'
        ctx['uses_map'] = True

        floors = [
            {
                'caption': f'{floor["name"]} ({floor["index"]})',
                'link': reverse('backoffice:map-floor-edit', kwargs={'pk': floor['pk']}),
            }
            for floor in MapFloor.objects.filter(conference=self.conference).order_by('index').values('pk', 'name', 'index')
        ]

        pois = []
        poi_count = 0
        ctx['sidebar'] = {
            'title': _('nav_map'),
            'items': [
                {
                    'caption': _('MapFloors'),
                    'children': floors,
                    'count': len(floors),
                    'add_link': reverse('backoffice:map-floor-create'),
                },
                {
                    'caption': _('MapPOIs'),
                    'children': pois,
                    'add_link': reverse('backoffice:map-poi-create'),
                },
            ],
        }

        for poi in MapPOI.objects.filter(conference=self.conference).values('id', 'visible', 'name').iterator():
            poi_count += 1
            pois.append(
                {
                    'caption': poi['name'] if poi['visible'] else format_html('<s>{}</s>', poi['name']),
                    'link': reverse('backoffice:map-poi-edit', kwargs={'pk': poi['id']}),
                }
            )
        pois.insert(
            0,
            {
                'caption': format_html('<i>({all})</i>', all=_('all')),
                'link': reverse('backoffice:map-poi-list'),
                'count': poi_count,
            },
        )

        # try to guess 'active' sidebar item
        guess_active_sidebar_item(self.request, ctx['sidebar']['items'], with_query_string=False)

        return ctx


class FloorListView(MapAdminMixin, ListView):
    template_name = 'backoffice/map_floor_list.html'

    model = MapFloor

    def get_queryset(self, *args, **kwargs):
        return MapFloor.objects.filter(conference=self.conference).order_by('index')


class FloorFormMixin(MapAdminMixin, SingleObjectTemplateResponseMixin):
    fields = ['name_de', 'name_en', 'index']
    template_name = 'backoffice/map_floor_form.html'
    model = MapFloor

    def get_queryset(self, *args, **kwargs):
        return MapFloor.objects.filter(conference=self.conference)

    def form_valid(self, form):
        messages.success(self.request, _('updated'))
        # TODO: ConferenceExportCache.signal_poi_modification(self.conference)
        return super().form_valid(form)

    def get_success_url(self):
        return reverse('backoffice:map-floor-edit', kwargs={'pk': self.object.id})


class FloorCreateView(FloorFormMixin, CreateView):
    def form_valid(self, form):
        form.instance.conference = self.conference
        return super().form_valid(form)


class FloorUpdateView(FloorFormMixin, UpdateView):
    pass


class POIListView(MapAdminMixin, ListView):
    template_name = 'backoffice/map_poi_list.html'

    model = MapPOI

    def get_queryset(self, *args, **kwargs):
        return MapPOI.objects.filter(conference=self.conference).order_by('name')


class POIFormMixin(MapAdminMixin, ModelFormMixin):
    model = MapPOI
    form_class = POIForm
    template_name = 'backoffice/map_poi_form.html'

    def get_queryset(self, *args, **kwargs):
        return MapPOI.objects.filter(conference=self.conference)

    def form_valid(self, form):
        messages.success(self.request, _('updated'))
        # TODO: ConferenceExportCache.signal_poi_modification(self.conference)
        return super().form_valid(form)

    def get_success_url(self):
        return reverse('backoffice:map-poi-edit', kwargs={'pk': self.object.id})


class POICreateView(POIFormMixin, CreateView):
    def form_valid(self, form):
        form.instance.conference = self.conference
        return super().form_valid(form)


class POIUpdateView(POIFormMixin, UpdateView):
    pass
