import logging

from django.contrib import messages
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.generic import DeleteView

from core.models.assemblies import Assembly
from core.models.projects import Project

from backoffice.views.mixins import ConferenceLoginRequiredMixin
from backoffice.views.projects.asssembly_projects import (
    AssemblyProjectCreateView,
    AssemblyProjectListView,
    AssemblyProjectUpdateView,
)
from backoffice.views.projects.self_organized_projects import (
    ProjectCreateView,
    SelfOrganizedProjectListView,
    SelfOrganizedProjectUpdateView,
)

logger = logging.getLogger(__name__)


class ProjectDeleteView(ConferenceLoginRequiredMixin, DeleteView):
    assembly: Assembly | None = None

    def get_success_url(self) -> str:
        if self.assembly is not None:
            return reverse('backoffice:assembly-projects', kwargs={'assembly': self.assembly.id})
        return reverse('backoffice:sos')

    def get_queryset(self):
        return Project.objects.manageable_by_user(self.conference, user=self.request.user)

    def get_log_data(self) -> dict[str, str]:
        project = self.get_object()
        if not Project.type_is(project):  # pragma: no cover
            raise ValueError('Object is not a Project')
        self.assembly = project.assembly
        log_data = {
            'project_type': _('AssemblyProject') if project.assembly else _('SelfOrganizedProject'),
            'project': project.name,
            'user': self.request.user,
            'owner': 'owner ' if project.owner else '',
            'assembly': f' on {project.assembly} ' if project.assembly else '',
        }
        return log_data

    def form_valid(self, form):
        log_data = self.get_log_data()
        response = super().form_valid(form)
        messages.success(self.request, _('Project__removed__success %(project_type)s %(project)s') % log_data)
        logger.info('removed %(project_type)s "%(project)s" %(assembly)sby %(owner)s%(user)s', log_data)
        return response

    def form_invalid(self, form):
        log_data = self.get_log_data()
        response = super().form_invalid(form)
        messages.error(self.request, _('Project__removal_failure %(project_type)s %(project)s') % log_data)
        logger.info('removed %(project_type)s "%(project)s" %(assembly)sby %(owner)s%(user)s', log_data)
        return response


__all__ = [
    'AssemblyProjectCreateView',
    'AssemblyProjectListView',
    'AssemblyProjectUpdateView',
    'ProjectCreateView',
    'ProjectDeleteView',
    'SelfOrganizedProjectListView',
    'SelfOrganizedProjectUpdateView',
]
