import collections
import datetime
from xml.etree import ElementTree

import requests
from requests_file import FileAdapter

from django.utils.dateparse import parse_datetime

from .base import BaseScheduleSupport, filter_additional_data, schedule_time_to_timedelta

s = requests.Session()
s.mount('file://', FileAdapter())


class ScheduleXMLSupport(BaseScheduleSupport):
    identifier = 'schedule-xml'
    readonly = True

    configuration_fields = None

    def ready(self):
        r = s.head(self.remote_url)
        return r.ok

    def fetch(self):
        """
        This method is the workhorse of the schedule support module:
        its job is to query upstream for the current set of data.

        It shall return a dictionary with keys 'rooms' and 'events',
        each containing a dictionary with entries mapping a source id
        to a dictionary which can be understood by Room.from_dict()
        and Event.from_dict() respectively.

        The hub will update events it already knows by the source id:
        all events need to have an unique but stable identifier, i.e.
        if the name of the event changes the identifier must not change!
        """

        schedule = ScheduleXML.from_url(self.remote_url)

        return {
            'version': schedule.version(),
            'rooms': {r['name']: r for r in schedule.rooms()},
            'events': {
                e.get('id'): {
                    'guid': e.get('guid'),
                    'slug': e.get('slug').split(f"{e.get('id')}-")[1][0:150].strip('-') or e.get('slug')[0:150].strip('-'),
                    'name': e.get('title'),
                    'language': e.get('language'),
                    'abstract': e.get('abstract') or '',
                    'description': e.get('description') or '',
                    'track': e.get('track'),
                    'room': e.get('room'),
                    'schedule_start': e.get('date'),
                    'schedule_duration': str(schedule_time_to_timedelta(e.get('duration'))),
                    'is_public': True,
                    'additional_data': filter_additional_data(e),
                }
                for e in schedule.events()
            },
        }


class ScheduleXML:
    """
    Schedule from XML document using etree, with inspirations from
      - https://github.com/pretalx/pretalx-downstream/blob/master/pretalx_downstream/tasks.py#L67
      - https://github.com/Zverik/schedule-convert/blob/master/schedule_convert/importers/frab_xml.py#L55
    """

    _schedule: ElementTree = None
    tz = None

    def __init__(self, tree):
        self._schedule = tree

    @classmethod
    def from_url(cls, url):
        r = s.get(url)

        if r.ok is False:
            raise Exception(f'Request failed, HTTP {r.status_code}.')

        schedule = ElementTree.fromstring(r.text)

        # Close the raw file handle if it's still open
        if hasattr(r, 'raw') and r.raw.closed is False:
            r.raw.close()

        return ScheduleXML(tree=schedule)

    def __getitem__(self, key):
        return self._schedule.find(key)

    def schedule(self):
        return self._schedule

    def version(self):
        v = self._schedule.find('version')
        return v.text if v is not None else None

    def days(self):
        return self._schedule.findall('day')

    def rooms(self):
        rooms = {}
        for day in self.days():
            for room in day.findall('room'):
                room_name = room.attrib['name']
                room_guid = room.attrib.get('guid')
                room_desc = room.attrib.get('description')
                rooms[room_name] = {'guid': room_guid, 'name': room_name, 'description': room_desc}

        return list(rooms.values())

    def events(self):
        for day in self.days():
            for room in day.findall('room'):
                for event in room.findall('event'):
                    e = Event(event)
                    e['id'] = event.attrib.get('id')
                    e['guid'] = event.attrib.get('guid')
                    yield e

    def __str__(self):
        return ElementTree.tounicode(self._schedule, pretty_print=True)


class Event(collections.abc.Mapping):
    _event: ElementTree = None
    start: datetime = None

    def __init__(self, tree, start_time=None):
        self._event = tree
        self._values = {}
        if start_time:
            self.start = start_time
        else:
            node = self._event.find('date')
            self.start = parse_datetime(node.text) if node is not None else None

    def __getitem__(self, key):
        if key in self._values:
            return self._values[key]

        item = self._event.find(key)
        return item.text if item is not None else None

    def __setitem__(self, key, value):
        self._values[key] = value

    def __iter__(self):
        # TODO
        return self._event.__iter__()

    def __len__(self):
        # TODO
        return len(self._event)

    def __lt__(self, other):
        return self.start < other.start

    def items(self):
        # TODO
        return self._event.items()
