from unittest.mock import Mock, patch

from django.test import TestCase

from core.integrations import BigBlueButtonIntegration, IntegrationError
from core.models import Assembly, Conference, PlatformUser, Room

BigBlueButton = BigBlueButtonIntegration('http://localhost/', 'asdf', 'https://localhost/end_meeting', None)


# from https://github.com/Grollicus/unittest_patterns/blob/master/unittest_patterns/__init__.py
class Pattern:
    def __req__(self, lhs):
        return self.__eq__(lhs)

    __hash__ = None


# from https://github.com/Grollicus/unittest_patterns/blob/master/unittest_patterns/__init__.py
class Any(Pattern):
    """Equals everything"""

    def __eq__(self, rhs):
        return True


class BigBlueButtonTest(TestCase):
    def setUp(self):
        self.conf = Conference(slug='conf', name='TestConf')
        self.conf.save()
        self.assembly = Assembly(name='TestAssembly', slug='asmbly', conference=self.conf)
        self.assembly.save()
        self.room = Room(
            conference=self.conf, assembly=self.assembly, name='Room 1', room_type=Room.RoomType.BIGBLUEBUTTON, backend_status=Room.BackendStatus.NEW
        )
        self.room.save()

    def test_create_room(self):
        with patch.object(BigBlueButton._session, 'get') as get_mock:
            get_mock.return_value.status_code = 404
            with self.assertRaises(IntegrationError), self.assertLogs('core.integrations.bigbluebutton'):
                BigBlueButton.create_room(self.room)
            self.assertEqual(self.room.backend_status, Room.BackendStatus.ERROR)
            get_mock.assert_called_once()
            get_mock.reset_mock()

            get_mock.return_value.status_code = 200
            get_mock.return_value.text = (
                '<response>'
                '<returncode>SUCCESS</returncode>'
                '<meetingID>Test</meetingID>'
                '<internalMeetingID>640ab2bae07bedc4c163f679a746f7ab7fb5d1fa-1531155809613</internalMeetingID>'
                '<parentMeetingID>bbb-none</parentMeetingID>'
                '<attendeePW>ap</attendeePW>'
                '<moderatorPW>mp</moderatorPW>'
                '<createTime>1531155809613</createTime>'
                '<voiceBridge>70757</voiceBridge>'
                '<dialNumber>613-555-1234</dialNumber>'
                '<createDate>Mon Jul 09 17:03:29 UTC 2018</createDate>'
                '<hasUserJoined>false</hasUserJoined>'
                '<duration>0</duration>'
                '<hasBeenForciblyEnded>false</hasBeenForciblyEnded>'
                '<messageKey>duplicateWarning</messageKey>'
                '<message>This conference was already in existence and may currently be in progress.</message>'
                '</response>'
            )
            self.room.backend_status = Room.BackendStatus.NEW
            self.room.save()
            with self.assertLogs('core.integrations.bigbluebutton'):
                BigBlueButton.create_room(self.room)
            get_mock.assert_called_once()
            self.assertEqual(self.room.backend_status, Room.BackendStatus.ACTIVE)
            self.assertEqual(self.room.backend_link, 'Test')
            self.assertEqual(
                self.room.backend_data,
                {
                    'attendeePW': 'ap',
                    'moderatorPW': 'mp',
                    'createTime': '1531155809613',
                    'close_secret': Any(),
                },
            )

    def test_remove_room(self):
        with patch.object(BigBlueButton._session, 'get') as get_mock:
            get_mock.return_value.status_code = 404
            self.room.backend_status = Room.BackendStatus.ACTIVE
            self.room.backend_link = 'Test'
            self.room.backend_data = {
                'attendeePW': 'ap',
                'moderatorPW': 'mp',
                'createTime': '1531155809613',
            }
            self.room.save()
            with self.assertRaises(IntegrationError), self.assertLogs('core.integrations.bigbluebutton'):
                BigBlueButton.remove_room(self.room)
            self.assertEqual(self.room.backend_status, Room.BackendStatus.ACTIVE)
            get_mock.assert_called_once()
            get_mock.reset_mock()

            get_mock.return_value.status_code = 200
            get_mock.return_value.text = (
                '<response>'
                '<returncode>SUCCESS</returncode>'
                '<messageKey>sentEndMeetingRequest</messageKey>'
                '<message>'
                'A request to end the meeting was sent.  Please wait a few seconds, and then use the getMeetingInfo or'
                ' isMeetingRunning API calls to verify that it was ended'
                '</message>'
                '</response>'
            )
            self.room.backend_status = Room.BackendStatus.ACTIVE
            self.room.save()
            BigBlueButton.remove_room(self.room)
            get_mock.assert_called_once()
            self.assertEqual(self.room.backend_status, Room.BackendStatus.INACTIVE)

    def test_room_status(self):
        self.room.backend_status = Room.BackendStatus.NEW
        self.room.backend_link = 'Demo Meeting'
        self.room.backend_data = {
            'attendeePW': 'ap',
            'moderatorPW': 'mp',
            'createTime': '1531155809613',
        }
        self.room.occupants = None
        self.room.save()

        with patch.object(BigBlueButton._session, 'get') as get_mock:
            BigBlueButton.room_status(self.room)
            get_mock.assert_not_called()

            self.room.backend_status = Room.BackendStatus.ACTIVE
            get_mock.return_value.status_code = 200
            get_mock.return_value.text = (
                '<response>'
                '<returncode>SUCCESS</returncode>'
                '<meetingName>Demo Meeting</meetingName>'
                '<meetingID>Demo Meeting</meetingID>'
                '<internalMeetingID>183f0bf3a0982a127bdb8161e0c44eb696b3e75c-1531155809613</internalMeetingID>'
                '<createTime>1531155809613</createTime>'
                '<createDate>Tue Jul 10 16:36:25 UTC 2018</createDate>'
                '<voiceBridge>70066</voiceBridge>'
                '<dialNumber>613-555-1234</dialNumber>'
                '<attendeePW>ap</attendeePW>'
                '<moderatorPW>mp</moderatorPW>'
                '<running>true</running>'
                '<duration>0</duration>'
                '<hasUserJoined>true</hasUserJoined>'
                '<recording>false</recording>'
                '<hasBeenForciblyEnded>false</hasBeenForciblyEnded>'
                '<startTime>1531240585239</startTime>'
                '<endTime>0</endTime>'
                '<participantCount>2</participantCount>'
                '<listenerCount>1</listenerCount>'
                '<voiceParticipantCount>1</voiceParticipantCount>'
                '<videoCount>1</videoCount>'
                '<maxUsers>20</maxUsers>'
                '<moderatorCount>1</moderatorCount>'
                '<attendees>'
                '<attendee>'
                '<userID>w_2wzzszfaptsp</userID>'
                '<fullName>stu</fullName>'
                '<role>VIEWER</role>'
                '<isPresenter>false</isPresenter>'
                '<isListeningOnly>true</isListeningOnly>'
                '<hasJoinedVoice>false</hasJoinedVoice>'
                '<hasVideo>false</hasVideo>'
                '<clientType>FLASH</clientType>'
                '</attendee>'
                '</attendees>'
                '<metadata />'
                '<isBreakout>false</isBreakout>'
                '</response>'
            )
            BigBlueButton.room_status(self.room)
            self.assertEqual(self.room.occupants, 2)
            get_mock.reset_mock()

            get_mock.return_value.status_code = 200
            get_mock.return_value.text = (
                '<response>'
                '<returncode>FAILED</returncode>'
                '<messageKey>notFound</messageKey>'
                '<message>We could not find a meeting with that meeting ID</message>'
                '</response>'
            )
            BigBlueButton.room_status(self.room)
            self.assertEqual(self.room.occupants, None)
            self.assertEqual(self.room.backend_status, Room.BackendStatus.INACTIVE)

    def test_join_room(self):
        user = PlatformUser(username='asdf')
        user.save()

        self.room.backend_status = Room.BackendStatus.ACTIVE
        self.room.backend_link = 'Test'
        self.room.backend_data = {
            'attendeePW': 'ap',
            'moderatorPW': 'mp',
            'createTime': '1531155809613',
        }
        self.room.save()

        with patch.object(BigBlueButton._session, 'get') as get_mock, self.assertLogs('core.integrations.bigbluebutton'):
            get_mock.side_effect = [
                Mock(
                    status_code=200,
                    text=(
                        '<response>'
                        '<returncode>FAILED</returncode>'
                        '<messageKey>notFound</messageKey>'
                        '<message>We could not find a meeting with that meeting ID</message>'
                        '</response>'
                    ),
                ),
                Mock(
                    status_code=200,
                    text=(
                        '<response>'
                        '<returncode>SUCCESS</returncode>'
                        '<meetingID>Test</meetingID>'
                        '<internalMeetingID>640ab2bae07bedc4c163f679a746f7ab7fb5d1fa-1531155809613</internalMeetingID>'
                        '<parentMeetingID>bbb-none</parentMeetingID>'
                        '<attendeePW>ap</attendeePW>'
                        '<moderatorPW>mp</moderatorPW>'
                        '<createTime>1531155809613</createTime>'
                        '<voiceBridge>70757</voiceBridge>'
                        '<dialNumber>613-555-1234</dialNumber>'
                        '<createDate>Mon Jul 09 17:03:29 UTC 2018</createDate>'
                        '<hasUserJoined>false</hasUserJoined>'
                        '<duration>0</duration>'
                        '<hasBeenForciblyEnded>false</hasBeenForciblyEnded>'
                        '<messageKey>duplicateWarning</messageKey>'
                        '<message>This conference was already in existence and may currently be in progress.</message>'
                        '</response>'
                    ),
                ),
                Mock(status_code=302, headers={'Location': 'https://yourserver.com/client/BigBlueButton.html?sessionToken=ai1wqj8wb6s7rnk0'}),
            ]
            url = BigBlueButton.join_room(self.room, user)
            self.assertEqual(url, 'https://yourserver.com/client/BigBlueButton.html?sessionToken=ai1wqj8wb6s7rnk0')
            self.assertEqual(self.room.backend_status, Room.BackendStatus.ACTIVE)
