import calendar
import json
import logging

from oauth2_provider.models import get_access_token_model
from oauth2_provider.views import AuthorizationView, ClientProtectedScopedResourceView

from csp.decorators import csp_update
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import TemplateView

from core.models import Conference

logger = logging.getLogger(__name__)


class SelectedConferenceContextMixin:
    def get_context_data(self, *args, **kwargs):
        ctx = super().get_context_data(*args, **kwargs)
        ctx['conference'] = Conference.objects.public_accessible().filter(pk=settings.SELECTED_CONFERENCE_ID)
        return ctx


@method_decorator(csp_update(form_action='*'), name='dispatch')
class OutOfBandDisplayTokenView(SelectedConferenceContextMixin, TemplateView):
    template_name = 'sso/out-of-band-display-token.html'


@method_decorator(csp_update(form_action='*'), name='dispatch')
class AuthorizeView(SelectedConferenceContextMixin, AuthorizationView):
    template_name = 'sso/authorize.html'


@method_decorator(csrf_exempt, name='dispatch')
class IntrospectTokenView(ClientProtectedScopedResourceView):
    """
    Implements an endpoint for token introspection based
    on RFC 7662 https://tools.ietf.org/html/rfc7662
    To access this view the request must pass a OAuth2 Bearer Token
    which is allowed to access the scope `introspection`.

    Heavily inspired from django-oauth2-toolkit's original implementation.
    """

    required_scopes = ['introspection']

    @staticmethod
    def get_token_response(query_user, token_value=None):
        try:
            token = get_access_token_model().objects.select_related('user', 'application').get(token=token_value)

        except ObjectDoesNotExist:
            return HttpResponse(content=json.dumps({'active': False}), status=401, content_type='application/json')

        if not token.is_valid():
            logger.debug('Token %s is invalid.', token)
            return HttpResponse(content=json.dumps({'active': False}), status=200, content_type='application/json')

        if not token.user or not token.user.is_active:
            logger.info('User %s in token %s is missing or inactive.', token.user, token.token)
            return HttpResponse(content=json.dumps({'active': False}), status=200, content_type='application/json')

        if not token.application:
            logger.warning('Token without associated application: %s', token)
            return HttpResponse(content=json.dumps({'active': False}), status=200, content_type='application/json')

        # prepare generic OAuth2 data
        data = {
            'active': True,
            'client_id': token.application.client_id,
            'exp': int(calendar.timegm(token.expires.timetuple())),
        }

        # send response as JSON
        return HttpResponse(content=json.dumps(data), status=200, content_type='application/json')

    def get(self, request, *args, **kwargs):
        """
        Get the token from the URL parameters.
        URL: https://example.com/introspect?token=mF_9.B5f-4.1JqM
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        return self.get_token_response(request.user, request.GET.get('token', None))

    def post(self, request, *args, **kwargs):
        """
        Get the token from the body form parameters.
        Body: token=mF_9.B5f-4.1JqM
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        return self.get_token_response(request.user, request.POST.get('token', None))
