__all__ = (
    'AssembliesAllView',
    'AssembliesView',
    'AssemblyView',
)

import logging
from typing import Any

from django.db.models import Count, Prefetch, Q, QuerySet
from django.shortcuts import redirect
from django.views.generic import DetailView

from core.models import (
    Assembly,
    AssemblyMember,
    Badge,
    Event,
    Project,
    Room,
    TagItem,
)
from core.views.list_views import AlphabeticalPaginatorView, PrefetchTagsMixin

from plainui.views.utils import (
    ConferenceRequiredMixin,
    event_filter,
    session_get_favorite_assemblies,
    session_get_favorite_events,
    session_get_favorite_projects,
)

logger = logging.getLogger(__name__)


class AssemblyView(ConferenceRequiredMixin, DetailView):
    model = Assembly
    template_name = 'plainui/assembly.html.j2'
    slug_url_kwarg = 'assembly_slug'
    slug_field = 'slug__iexact'

    def get_queryset(self) -> QuerySet[Any]:
        return Assembly.objects.conference_accessible(self.conf).prefetch_related(
            Prefetch(
                'badges',
                to_attr='public_badges',
                queryset=Badge.objects.conference_accessible(self.conf).order_by('name'),
            ),
            Prefetch(
                'events',
                to_attr='public_events',
                queryset=Event.objects.conference_accessible(self.conf).exclude(kind=Event.Kind.SELF_ORGANIZED).order_by('schedule_start'),
            ),
            Prefetch(
                'members',
                to_attr='public_members',
                queryset=AssemblyMember.objects.conference_accessible(self.conf),
            ),
            Prefetch(
                'projects',
                to_attr='public_projects',
                queryset=Project.objects.conference_accessible(self.conf).order_by('name'),
            ),
            Prefetch(
                'rooms',
                to_attr='public_rooms',
                queryset=Room.objects.conference_accessible(self.conf).order_by('official_room_order', 'name'),
            ),
            Prefetch(
                'tags',
                to_attr='public_tags',
                queryset=TagItem.objects.conference_accessible(self.conf),
            ),
        )

    def get(self, request, *args, **kwargs):
        self.object = self.assembly = self.get_object()
        if self.conf.self_organized_sessions_assembly == self.assembly:
            return redirect('plainui:sos')
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        context['conf'] = self.conf

        assembly = self.assembly
        favorites = session_get_favorite_assemblies(self.request.session, self.request.user)
        context['assembly'] = assembly
        context['is_favorite'] = str(assembly.id) in favorites
        context['is_favorite_events'] = session_get_favorite_events(self.request.session, self.request.user)
        context['is_favorite_projects'] = session_get_favorite_projects(self.request.session, self.request.user)
        user = self.request.user
        context['owned_badges'] = user.badges.all().values_list('badge__pk', flat=True) if user.is_authenticated else []

        context['report_info'] = {'enabled': True, 'lookup_key': assembly.slug}
        context['fav_info'] = {'type': 'assembly', 'id': assembly.id, 'is': str(assembly.id) in favorites}

        return context


class AssembliesAllView(ConferenceRequiredMixin, PrefetchTagsMixin, AlphabeticalPaginatorView):
    model = Assembly
    template_name = 'plainui/assemblies_all.html.j2'
    context_object_name = 'assemblies'
    filters = {
        'name': ('name__icontains', 'str'),
        'official': ('is_official', 'bool'),
        'on_site': ('is_physical', 'bool'),
        'tags': ('tags__tag__slug__in', 'list'),
    }
    ordering = ['name']

    def get_queryset(self, queryset: QuerySet[Assembly] | None = None) -> QuerySet[Assembly]:
        public_events = Event.objects.conference_accessible(self.conf)
        public_projects = Project.objects.conference_accessible(self.conf)
        queryset = queryset if queryset else Assembly.objects.conference_accessible(self.conf)
        queryset = queryset.annotate(
            public_events_count=Count('events', filter=Q(events__in=public_events), distinct=True),
            public_projects_count=Count('projects', Q(projects__in=public_projects), distinct=True),
        )
        queryset = self.add_tag_annotations(queryset)
        return super().get_queryset(queryset)

    def get_context_data(self, **kwargs):
        return {
            **super().get_context_data(**kwargs),
            'conf': self.conf,
            'my_favorite_assemblies': session_get_favorite_assemblies(self.request.session, self.request.user),
            'report_info': {'enabled': True},
        }


class AssembliesView(AssembliesAllView):
    template_name = 'plainui/assemblies.html.j2'

    def get_context_data(self, **kwargs):
        # TODO: recommended events.
        # TODO: Implement suggestions based on liked assemblies
        return {
            **super().get_context_data(**kwargs),
            'events_upcoming': event_filter(self.request.user, self.conf, upcoming=True),
            'is_favorite_events': session_get_favorite_events(self.request.session, self.request.user),
            'report_info': {'enabled': True},
        }
