__all__ = (
    'LoginView',
    'LogoutView',
    'PasswordChangeView',
    'PasswordResetCompleteView',
    'PasswordResetConfirmView',
    'PasswordResetDoneView',
    'PasswordResetView',
    'RegistrationActivationView',
    'RegistrationDoneView',
    'RegistrationView',
    'TokenPasswordResetView',
)

from typing import Any

from django.conf import settings
from django.contrib import messages
from django.contrib.auth import views as auth_views
from django.http import HttpResponseRedirect
from django.urls import reverse, reverse_lazy
from django.utils.decorators import method_decorator
from django.utils.translation import gettext
from django.views.decorators.debug import sensitive_post_parameters
from django.views.generic.base import TemplateView
from django.views.generic.edit import FormView

from core.views import BaseLoginView, BasePasswordResetConfirmView, BasePasswordResetView, BaseRegistrationActivationView, BaseRegistrationView

from plainui.forms import (
    TokenPasswortResetForm,
)
from plainui.views.utils import (
    ConferenceRequiredMixin,
)


class LoginView(ConferenceRequiredMixin, BaseLoginView):
    template_name = 'plainui/login.html.j2'
    next_page = 'plainui:index'
    require_conference_member = False
    require_login = False
    only_published = False

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['conf'] = self.conf
        context['hide_header'] = True
        context['disable_share'] = True
        return context

    def get_form_kwargs(self) -> dict[str, Any]:
        return {**super().get_form_kwargs(), 'with_theme_switcher': settings.PLAINUI_THEME_SUPPORT}


class RegistrationView(ConferenceRequiredMixin, BaseRegistrationView):
    template_name = 'plainui/signup.html.j2'
    success_url = reverse_lazy('plainui:signup_done')
    activation_url = 'plainui:signup_activate'
    require_conference_member = False
    require_login = False
    only_published = False

    def get_form_kwargs(self) -> dict[str, Any]:
        return {**super().get_form_kwargs(), 'conference': self.conf}

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['conf'] = self.conf
        context['hide_header'] = True
        context['disable_share'] = True
        return context


class RegistrationDoneView(ConferenceRequiredMixin, TemplateView):
    template_name = 'plainui/signup_done.html.j2'
    require_conference_member = False
    require_login = False
    only_published = False

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['conf'] = self.conf
        context['hide_header'] = True
        context['disable_share'] = True
        return context


class RegistrationActivationView(ConferenceRequiredMixin, BaseRegistrationActivationView):
    require_conference_member = False
    require_login = False
    only_published = False
    redirect_target_failure = 'plainui:signup'
    redirect_target_success = 'plainui:index'

    def get_context_data(self, **kwargs) -> dict[str, Any]:
        return {
            **super().get_context_data(**kwargs),
            'disable_share': True,
        }


class PasswordChangeView(ConferenceRequiredMixin, auth_views.PasswordChangeView):
    require_user = True
    require_conference_member = False
    only_published = False
    template_name = 'plainui/password_change.html.j2'

    @method_decorator(sensitive_post_parameters())
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)

    def get_context_data(self, **kwargs):
        return {
            **super().get_context_data(**kwargs),
            'conf': self.conf,
            'disable_share': True,
        }

    def get_success_url(self):
        messages.success(self.request, gettext('Password changed successfully.'))
        return reverse('plainui:userprofile')


class PasswordResetView(ConferenceRequiredMixin, BasePasswordResetView):
    require_conference_member = False
    require_login = False
    only_published = False
    success_url = reverse_lazy('plainui:password_reset_done')
    template_name = 'plainui/registration/password_reset_form.html.j2'
    extra_email_context = {'reset_url': 'plainui:password_reset_confirm'}

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['conf'] = self.conf
        context['hide_header'] = True
        context['disable_share'] = True
        return context


class PasswordResetDoneView(ConferenceRequiredMixin, auth_views.PasswordResetDoneView):
    template_name = 'plainui/password_reset_done.html.j2'
    require_conference_member = False
    require_login = False
    only_published = False

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['conf'] = self.conf
        context['hide_header'] = True
        context['disable_share'] = True
        return context


class PasswordResetConfirmView(ConferenceRequiredMixin, BasePasswordResetConfirmView):
    template_name = 'plainui/password_reset_confirm.html.j2'
    require_conference_member = False
    require_login = False
    only_published = False
    success_url = reverse_lazy('plainui:password_reset_complete')
    failure_url = reverse_lazy('plainui:password_reset')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['conf'] = self.conf
        context['hide_header'] = True
        context['disable_share'] = True
        return context


class PasswordResetCompleteView(ConferenceRequiredMixin, auth_views.PasswordResetCompleteView):
    template_name = 'plainui/password_reset_complete.html.j2'
    require_conference_member = False
    require_login = False
    only_published = False

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['conf'] = self.conf
        context['hide_header'] = True
        context['disable_share'] = True
        return context


class TokenPasswordResetView(ConferenceRequiredMixin, FormView):
    template_name = 'plainui/token_password_reset.html.j2'
    require_conference_member = False
    require_login = False
    only_published = False
    form_class = TokenPasswortResetForm

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        context['conf'] = self.conf
        context['disable_share'] = True
        return context

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['conf'] = self.conf
        kwargs['user'] = self.request.user
        return kwargs

    def form_valid(self, form):
        form.save()
        messages.success(self.request, gettext('Passwort successfully reset!'))
        return HttpResponseRedirect(reverse('plainui:login'))


class LogoutView(auth_views.LogoutView):
    next_page = 'plainui:login'
